#include "Constantes.h"
#include "Bebida.h"

// Constructor
Bebida::Bebida(const string &name) : Atomic(name),
		Pedido_in(addInputPort("Pedido_in")),
		Agua_in(addInputPort("Agua_in")),
		Bebida_out(addOutputPort("Bebida_out")),
		Notif_out(addOutputPort("Notif_out")) {
}

// Inicializacion
Model& Bebida::initFunction() {
	this->estado = Normal;
	this->BebidaPreparada = 0;

	passivate();

	return *this;
}

// Funcion de transicion externa
Model& Bebida::externalFunction(const ExternalMessage& msg) {

	if (msg.port() == Pedido_in) {

		if (estaPasivo()) {
			if (this->estado == Normal) {
				this->estado = AvisandoListo;
				this->BebidaPreparada = (int) msg.value();
				holdIn(0);
			}
		}

	} else
	if (msg.port() == Agua_in) {

		if (estaPasivo()) {
			if ((int) msg.value() == INICIO_AGUA) {
				if (this->estado == EsperandoAgua) {
					this->estado = PreparandoBebida;
					passivate();
				}
			} else
			if ((int) msg.value() == FIN_AGUA) {
				if (this->estado == PreparandoBebida) {
					this->estado = EntregandoBebida;
					holdIn(0);
				}
			}
		}
	}

	return *this;
}


// Funcion de transicion interna
Model& Bebida::internalFunction(const InternalMessage& msg) {

	switch (this->estado) {
		case AvisandoListo:
			this->estado = EsperandoAgua;
			passivate();
		break;
		case EntregandoBebida:
			this->estado = Normal;
			passivate();
		break;

		default:
		break;
	}

	return *this;
}

// Funcion de salida
Model& Bebida::outputFunction(const CollectMessage& msg) {

	switch (this->estado) {

		case AvisandoListo:
			// Aviso a la UC el cafe consumido
			sendOutput(msg.time(), Notif_out, (Value) -this->BebidaPreparada);
			break;

		case EntregandoBebida:
			sendOutput(msg.time(), Bebida_out, (Value) this->BebidaPreparada);

		default:
			break;
	}

	return *this;
}

// Funcion holdIn propia
void Bebida::holdIn(const VTime& t) {
	Atomic::holdIn(AtomicState::active, t);
}

bool Bebida::estaPasivo() {
	return state() == AtomicState::passive;
}
