#include <iostream>
#include "UC.h"
#include "Receta.h"
#include "Constantes.h"


// Constructor
UC::UC(const string &name) : Atomic(name),
		Recarga(addInputPort("Recarga")),
		RecibePedido_in(addInputPort("RecibePedido_in")),
		FichaLista_in(addInputPort("FichaLista_in")),
		Notif_in(addInputPort("Notif_in")),
		
		//puertos de salida
		Habilita_out(addOutputPort("Habilita_out")),
		Confirma_out(addOutputPort("Confirma_out")),
		PedidoCafe_out(addOutputPort("PedidoCafe_out")),
		PedidoChoc_out(addOutputPort("PedidoChoc_out")),
		PedidoLeche_out(addOutputPort("PedidoLeche_out")),
		PedidoAgua_out(addOutputPort("PedidoAgua_out")),	
		PedidoVaso_out(addOutputPort("PedidoVaso_out")),	
		EnviarMensaje_out(addOutputPort("EnviarMensaje_out")) {
		
}

// Inicializacion
Model& UC::initFunction() {

	this->estado = Normal;
	this->CantLeche=0;
	this->CantChocolate=0;
	this->CantCafe=0;
	this->CantVasos=0;
	
	//Creo las recetas. Ingredientes: Chocolate, Leche, Cafe.
	this->RecetaLagrima.CrearReceta(0, 7, 3);
	this->RecetaCafe.CrearReceta(0, 0, 10);
	this->RecetaCafeConLeche.CrearReceta(0, 5, 5);
	this->RecetaChocolate.CrearReceta(5, 0, 5);
	this->Mensaje= 0;

	return *this;
}


// Funcion de transicion externa
Model& UC::externalFunction(const ExternalMessage& msg) {

	if (msg.port() == Recarga){
		
		if (this->estado == Normal && (int) msg.value() == 0) {
			this->estado = Recargando;
			passivate();
		} else 
			if (this->estado == Recargando && (int) msg.value() == 1) {
				//Por ahora las recargas setean todos los ingredientes a 500.
				this->CantLeche= 500;
				this->CantChocolate= 500;
				this->CantCafe= 500;
				this->CantVasos= 200;
				this->estado = IniciandoCiclo;
				holdIn(0);
			}
	} else 
	
	if (msg.port()==FichaLista_in){
		if (this->estado == Normal){
			this->Mensaje= Elija_una_bebida;
			this->estado= RecibiendoPedido;

			holdIn(0);
		} else
		if (this->estado == RecibiendoPedido) {
			this->estado = Normal;
			passivate();
		}
	
	} else 
	if((msg.port()==RecibePedido_in) && (this->estado==RecibiendoPedido)) {

		switch ((int) msg.value()) {
		
			case Chocolate:
				this->sabor= RecetaChocolate;
			break;
			case Lagrima:
				this->sabor= RecetaLagrima;
			break;
			case CafeConLeche:
				this->sabor= RecetaCafeConLeche;
			break;
			case Cafe:
				this->sabor= RecetaCafe;
			break;
			default:
				cerr << "DEBUG: Receta desconocida @ UC" << endl;
			break;

		}
		
		//Chequea disponibilidad de ingredientes para el sabor elegido.
		if (CantChocolate>=sabor.Chocolate() &&  CantCafe>=sabor.Cafe() && CantLeche>=sabor.Leche() && CantVasos>0) {
			
			this->Mensaje = Procesando_Pedido;
			this->estado= ConfirmandoPedido;
			holdIn(0);
			
		}	
		else {

			this->Mensaje = No_Se_Puede;
			this->estado = RechazandoPedido;
			holdIn(0);

		};
		
	} else 
	
	if (msg.port() == Notif_in){
		
		if (this->estado==EsperandoNotifVaso) {	//***
			cerr << "DEBUG: Notificacion de vaso recibida @ uc" << endl;
			this->estado= CargandoCafe;
			this->CantVasos--;
			holdIn(0);
		} else

		if (this->estado==EsperandoNotifCafe) {	//***
			cerr << "DEBUG: Notificacion de cafe recibida @ uc" << endl;
			this->estado= SirviendoCafe;
			this->CantCafe -= sabor.Cafe();
			holdIn(0);
		} else

		if (this->estado==EsperandoNotifLeche) {	//***
			cerr << "DEBUG: Notificacion de leche recibida @ uc" << endl;
			this->estado= SirviendoLeche;
			this->CantLeche -= sabor.Leche();
			holdIn(0);
		} else

		if (this->estado==EsperandoNotifChocolate) {	//***
			cerr << "DEBUG: Notificacion de chocolate recibida @ uc" << endl;
			this->estado= SirviendoChocolate;
			this->CantChocolate -= sabor.Chocolate();
			holdIn(0);
		} 

	}
	return *this;
}


// Funcion de transicion interna
Model& UC::internalFunction(const InternalMessage& msg) {

	switch (this->estado) {
		
		case IniciandoCiclo:
			this->estado = Normal;
			passivate();
		break;
		case RecibiendoPedido:
			passivate();
		break;
		
		case RechazandoPedido:
			cerr << "DEBUG: Rechazando pedido @ UC" << endl;
			this->estado= Normal;
		break;
		case ConfirmandoPedido:
			cerr << "DEBUG: Confirmando pedido @ UC" << endl;
			this->estado= CargandoVaso;
			holdIn(0);
		break;
		
		case CargandoVaso:
			cerr << "DEBUG: Esperando notificacion vaso" << endl;
			this->estado = EsperandoNotifVaso;
			passivate();
		break;
		
		case CargandoCafe:
			this->estado= EsperandoNotifCafe;
			passivate();
		break;
		case SirviendoCafe: {
			this->estado= CargandoLeche;
			VTime v(0, 0, sabor.Cafe() +DEMORA, 0);
			holdIn(v);
		} break;
		
		case CargandoLeche:	
			this->estado= EsperandoNotifLeche;
			passivate();
		break;
		case SirviendoLeche: {
			this->estado= CargandoChocolate;
			VTime v(0, 0, sabor.Leche() +DEMORA, 0);
			holdIn(v);
		}
		break;

		case CargandoChocolate:	
			this->estado= EsperandoNotifChocolate;
			passivate();
		break;
		case SirviendoChocolate: {
			this->estado= EntregandoPedido;
			this->Mensaje = Retire_Vaso;
			VTime v(0, 0, sabor.Chocolate() +DEMORA, 0);
			holdIn(v);
		} break;
		
		case EntregandoPedido:
			this->Mensaje = Inserte_Ficha;
			this->estado = IniciandoCiclo;
			holdIn(0);
		break;

		default:
		break;

	}

	return *this;
}

// Funcion de salida
Model& UC::outputFunction(const CollectMessage& msg) {

	switch (this->estado) {

		case IniciandoCiclo:
			sendOutput(msg.time(), EnviarMensaje_out, (Value) this->Mensaje);

		case RecibiendoPedido:
			sendOutput(msg.time(), EnviarMensaje_out,(Value) this->Mensaje);
			sendOutput(msg.time(), Habilita_out, HABILITAR_BOTONERA);
		break;
		
		//Confirmacin o rechazo del pedido.
		case ConfirmandoPedido:
			sendOutput(msg.time(), Confirma_out,(Value) COMPRAR);
			sendOutput(msg.time(), EnviarMensaje_out,(Value) this->Mensaje);
			sendOutput(msg.time(), Habilita_out, DESHABILITAR_BOTONERA);
		break;

		case RechazandoPedido: 		
			sendOutput(msg.time(), Confirma_out,(Value) RECHAZAR);
			sendOutput(msg.time(), EnviarMensaje_out,(Value) this->Mensaje);
			//deshabilitar la botonera
			sendOutput(msg.time(), Habilita_out, (Value) DESHABILITAR_BOTONERA);
		break;
		
		//Fase Vaso.
		case CargandoVaso:
			cerr << "DEBUG: pedido de vaso @ uc" << endl;
			sendOutput(msg.time(), PedidoVaso_out,(Value) 1);
		break;
				
		//Fase Cafe.
		case CargandoCafe:
			sendOutput(msg.time(),PedidoCafe_out,(Value) this->sabor.Cafe());
		break;

		case SirviendoCafe:
			sendOutput(msg.time(),PedidoAgua_out,(Value) this->sabor.Cafe());
		break;
		
		//Fase Chocolate.
		case CargandoChocolate:
			sendOutput(msg.time(),PedidoChoc_out, (Value)this->sabor.Chocolate());
		break;

		case SirviendoChocolate:
			sendOutput(msg.time(),PedidoAgua_out, (Value)this->sabor.Chocolate()); 
		break;
		
		//Fase Leche.
		case CargandoLeche:
			sendOutput(msg.time(), PedidoLeche_out, (Value)this->sabor.Leche());
		break;

		case SirviendoLeche:
			sendOutput(msg.time(), PedidoAgua_out, (Value)this->sabor.Leche()); 
		break;
		
		case EntregandoPedido: 		
			sendOutput(msg.time(), EnviarMensaje_out,(Value) this->Mensaje); //***
		break;

		default:
		break;

	
	}

	return *this;
}

// Funcion holdIn propia
void UC::holdIn(const VTime& t) {
	Atomic::holdIn(AtomicState::active, t);
}

